{% if router == 'fastapi' %}
    {% include '_fastapi_app.py.jinja' %}
{% elif router == 'starlette' %}
    {% include '_starlette_app.py.jinja' %}
{% elif router == 'blacksheep' %}
    {% include '_blacksheep_app.py.jinja' %}
{% elif router == 'litestar' %}
    {% include '_litestar_app.py.jinja' %}
{% elif router == 'esmerald' %}
    {% include '_esmerald_app.py.jinja' %}
{% elif router == 'lilya' %}
    {% include '_lilya_app.py.jinja' %}
{% elif router == 'quart' %}
    {% include '_quart_app.py.jinja' %}
{% elif router == 'falcon' %}
    {% include '_falcon_app.py.jinja' %}
{% elif router == 'sanic' %}
    {% include '_sanic_app.py.jinja' %}
{% endif %}
